#################################s#o#q######################################
## by Arthur Chan @ soqhome.com
#############################################################################

#############################################################################


from Products.Archetypes.public import BaseFolderSchema, Schema
from Products.Archetypes.public import BaseFolder, registerType

from Products.CMFFolder.config import *

from Products.Archetypes.public import BooleanField, FixedPointField, IntegerField, LinesField, StringField, TextField
from Products.Archetypes.public import BooleanWidget, DecimalWidget, IntegerWidget, LinesWidget, SelectionWidget, StringWidget, TextAreaWidget


from Products.CMFCore import CMFCorePermissions
from AccessControl import ClassSecurityInfo

#from Persistence import Persistent

#from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from AccessControl import getSecurityManager, SpecialUsers
from Acquisition import aq_base, aq_parent, Acquired, Implicit
import string


#############################################################################
##                             Setting                                     ##
#############################################################################

schema = BaseFolderSchema +  Schema((
    StringField('title',
                searchable=1,
                required=1,
                ),
    BooleanField('within_cmffolder',
              default='1',
              mode='r',
              ),
    LinesField('left_slots',
              mode='r',
              accessor='left_slots'
              ),
    LinesField('custom_left_slots',
              widget=LinesWidget(
                  label="left slots",
                  description="leave blank if you want to acquire setting; type 'empty' if you do not use any slot.",
                  ),
              ),
    LinesField('right_slots',
              mode='r',
              accessor='right_slots'
              ),
    LinesField('custom_right_slots',
              widget=LinesWidget(
                  label="right slots",
                  description="leave blank if you want to acquire setting; type 'empty' if you do not use any slot.",
                  ),
              ),
    IntegerField('max_quota_bytes',
              default='0',
              widget=IntegerWidget(
                  label="Quota Size in Bytes",
                  description="(0 for no limit)",
                  ),
              ),
    IntegerField('max_quota_objects',
              default='0',
              widget=IntegerWidget(
                  label="Maximum number of Objects",
                  description="(0 for unlimited)",
                  ),
              ),
    IntegerField('max_object_size',
              default='0',
              widget=IntegerWidget(
                  label="Maximum Object Size in Bytes",
                  DecimalWidget="(0 for unlimited)",
                  ),
              ),
    BooleanField ('acquire_style',
              default=None,
              widget=BooleanWidget(label="Acquire Custom style",
                  description="De-select it only if you want to create your own " + \
                              "custom style sheet for this folder. " + \
                              "Otherwise, the acquired custom style will be used."
                 ),
        ),
    TextField('css_fields',
              default='',
              visible=1,
              widget=TextAreaWidget(label="Cascading Style Sheet",
                         description='Input your css code for this folder here'),
              ),
    BooleanField ('is_allow_nested',
              write_permission=CMFCorePermissions.ModifyPortalContent,
              default='1',
              mutator='setIs_allow_nested',
              widget=BooleanWidget(label="Allow Nested",
                  description="Allow creating CMFFolder insider another CMFFolder"
                 ),
        ),
    ))


#class CMFFolder(BaseFolder, Persistent, Implicit):
class CMFFolder(BaseFolder):
    """
        CMFFolder has quota count and assistant tools
    """
    security = ClassSecurityInfo()
    
    schema = schema
    archetype_name = "CMF Folder"
    
    #allow_discussion = 1
    
    global_allow = 1
    
    
    filter_content_types = 0
    security = ClassSecurityInfo()

    
    actions = ({'id': 'view',
                'name': 'View',
                'action': 'string:folder_listing',
                'permissions': (CMFCorePermissions.View,),
                'category'      : 'object' },
                 )
        
    #def __init__(self, id, title="", quota=0, maxFileSize=0, maxNumberOfFiles=0,
    #             REQUEST=None):
    #    self.id=id
    #    self.title=title
    #    self._quota_bytes=int(quota)
    #    self._quota_objects=int(maxNumberOfFiles)
    #    self._quota_maxsize=int(maxFileSize)
    #
    #    ##
    #    ## current accounting
    #    self._quota_filecount = 0
    #    self._quota_size = 0
    #
    #    self._allow_nested = 0

    security.declareProtected( CMFCorePermissions.View, 'left_slots' )        
    def left_slots(self):
        val = self.getCustom_left_slots()
        if not val:
            parent = self.aq_parent
            if hasattr(parent, 'left_slots'):
                ls = parent.left_slots
                if callable(ls):
                    return ls()
                else:
                    return ls
            else:
                return []
        elif 'empty' in val:
            return []
        else:
            return val
    
    security.declareProtected( CMFCorePermissions.View, 'right_slots' )        
    def right_slots(self):
        val = self.getCustom_right_slots()
        if not val:
            parent = self.aq_parent
            if hasattr(parent, 'right_slots'):
                rt = parent.right_slots
                if callable(rt):
                    return rt()
                else:
                    return rt
            else:
                return []
        elif 'empty' in val:
            return []
        else:
            return val

    security.declareProtected( CMFCorePermissions.ModifyPortalContent, 'setIs_allow_nested' )        
    def setIs_allow_nested(self, value):
        schema = self.Schema()
        field = schema.get('is_allow_nested', None)
        if value not in [1,'1']:
            if self.check_nested():
                 raise CMFFolderNestedException, \
                      "You already has nested CMFFolder within this folder, please remove it first before setting the folder to non-nested."

        field.set(instance=self, value=value)
        
    
    #############################################################################
    ##                             Utils                                       ##
    #############################################################################

    security.declarePublic('check_nested')
    def check_nested(self):
        items= self.portal_catalog(meta_type='CMFFolder', path=self.absolute_url('relative'))
        if len(items)>1:   #self is included
            return 1
        else:
            return 0
        

   
    #def _check_quota(self, filechange, sizechange=0, objsize=0):
    #    """ Check if the change in usage violates the quota settings """
    #    #
    #    # print "I'm a quota folder: ", filechange, sizechange, objsize
    #    # print "Current quota: ", self._quota_filecount, self._quota_size 
    #
    #    #
    #    # The emergency user can create *some* objects (i.e. userfolders).
    #    # make sure this never gives a QuotaExceeded exception
    #
    #    # get tools
    #    #catalog = self.portal_catalog
    #    #membership = self.portal_membership
    #    
    #    #items= self.portal_catalog(path=self.absolute_url('relative'))
    #    _quota_filecount = self.get_current_count()
    #    _quota_size = self.get_current_size() 
    #    quota_objects = self.getMax_quota_objects()
    #    quota_bytes = self.getMax_quota_bytes()
    #    quota_maxsize = self.getMax_quota_size()
    #    emergency = 0
    #    user=getSecurityManager().getUser()
    #    
    #    if (SpecialUsers.emergency_user and
    #        aq_base(user) is SpecialUsers.emergency_user):
    #        LOG('CMFFolder', INFO, 
    #            "Emergency user detected - not enforcing quota")
    #        emergency = 1
    #    
    #    if not emergency and \
    #       filechange > 0 and \
    #       _quota_filecount + filechange > quota_objects \
    #                                          > 0:
    #        raise QuotaExceededException, \
    #              "Too many files, this operation would require a quotum " + \
    #              "of %d files, your current quotum is %d files" % \
    #              (_quota_filecount + filechange, quota_objects)
    #              
    #    if not emergency and \
    #       sizechange > 0 and \
    #       _quota_size + sizechange > quota_bytes > 0:
    #        raise QuotaExceededException, \
    #             "Too much space, this operation would require a " + \
    #             "quotum of %d bytes, your current quotum is %d bytes" % \
    #             (_quota_size + sizechange, quota_bytes)
    #             
    #    if not emergency and objsize > quota_maxsize > 0:
    #        raise QuotaExceededException, \
    #              "The object is too large. It requires " + \
    #              "%s bytes, your maximum object size limit is %d bytes" % \
    #              (objsize, quota_maxsize)
    #
    #    #self._quota_filecount = self._quota_filecount + filechange
    #    #self._quota_size = self._quota_size + sizechange
    #
    #    ##
    #    ## This should not happen... 
    #    path = string.join(self.getPhysicalPath(), "/")
    #    if _quota_filecount < 0:
    #        LOG('CMFFolder', WARNING, "Negative quota filecount in %s" % path)
    #        #if not self._debug:
    #           # _quota_filecount = 0
    #    if _quota_size < 0:
    #        LOG('CMFFolder', WARNING, "Negative quota size in %s" % path)
    #        #if not self._debug:
    #         #   self._quota_size = 0
    #
    #    # print "New quota: ", self._quota_filecount, self._quota_size     

        



    
    
    security.declarePublic('check_over_count' )
    def check_over_count(self):
        quota = self.getMax_quota_objects()
        if not quota:
            return 0
        quota = int(quota)
        return self.getCurrentCount() > quota

    security.declarePublic('check_over_size' )
    def check_over_size(self):
        quota = self.getMax_quota_bytes()
        if not quota:
            return 0
        quota = int(quota)
        return self.getCurrentSize() > quota

    security.declarePublic('check_object_over_size' )
    def check_object_over_size(self):
        quota = self.getMax_object_size()
        if not quota:
            return 0
        quota = int(quota)
        path = self.getCurrentMaxObjectPath()
        if not path:
            return 0
        obj = self.restrictedTraverse(path)
        
        return obj.get_size() > quota

    security.declarePublic('getCurrentCount' )
    def getCurrentCount(self):
        return len(self.portal_catalog(path=self.absolute_url('relative')  )) -1  #(minus self)


    security.declarePublic('getCurrentMaxObjectPath')
    def getCurrentMaxObjectPath(self):
        items= self.portal_catalog(path=self.absolute_url('relative') )
        maxsize=0
        maxobjpath =''
        for i in items:
            o = i.getObject()
            if o <> self:
                size =o.get_size()
                
                if size > maxsize:
                    maxsize=size
                    maxobjpath = o.absolute_url('relative')
        return maxobjpath

    security.declarePublic('getCurrentSize' )
    def getCurrentSize(self):
        items= self.portal_catalog(path=self.absolute_url('relative') )
        sum = 0
        for i in items:
            o = i.getObject()
            if o <> self:
                sum = sum +  o.get_size()
                
        return sum

    #def getObjectSize(indexedObj, object):
    #    """ 
    #        attempt to determine number of size and number of subobjects of object 
    #    """
    #    ##
    #    ## ZPT doesn't return the size of the unrendered content it seems, 
    #    ## which leads to
    #    ## very inconsistent sizes. Let's hope this fixes it...
    #    obj = aq_base(object)
    #    sum = 0
    #    sum = sum + len(indexedObj.SearchableText())
    #    
    #    if hasattr(obj, "_get_usage"):
    #        tmp= len(obj._get_usage())
    #        if tmp > 0:
    #            sum = sum + tmp
    #    elif hasattr(obj, "get_size"):
    #        ##
    #        ## Some products (such as PhotoFolders Photo) raise an exception
    #        ## when requested their size at init time
    #        try:
    #            tmp = obj.get_size()
    #            if tmp > 0:
    #                sum = sum + tmp
    #        except: # can be any kind of exception
    #            LOG('CMFFolderProduct', BLATHER, 'Failed to get size for %s/%s' % \
    #                 (getid(obj.id), obj.meta_type))
    #    elif hasattr(obj, "getSize"):
    #        ## getSize is deprecated, but if it's all there is...
    #        tmp = obj.getSize()
    #        if tmp > 0:
    #            sum = sum + tmp
    #    else:
    #        LOG('CMFFolderProduct', BLATHER,  "Can't get size for %s/%s" % \
    #            (getid(obj.id), obj.meta_type))
    #    return sum
    #
    #
    
    security.declarePublic( 'getFileSizeStr' )
    def getFileSizeStr(self, bytes):
    
        try:
            
            """
            Format a number of bytes as bytes, K, or MB.
            """
            megs=bytes / 1048576.0
            if megs > 1:
                return "%.2f MB" % megs
            k=bytes / 1024.0
            if k > 1:
                return "%d K" % k
            return "%d bytes" % bytes
        except:
            return None

    def manage_afterAdd(self, item, container):
        # print "afterAdd %s %s"%(str(item), str(container))
        parent = self.aq_parent
        while hasattr(parent, 'aq_parent'):
            if hasattr(parent.aq_explicit, 'getIs_allow_nested'):
                allow_nested= parent.aq_explicit.getIs_allow_nested()
                if not allow_nested:
                    raise CMFFolderNestedException, \
                      "Cannot add a CMFFolder inside another CMFFolder"
            parent = parent.aq_parent
            
        # maybe we can check_over_size and check_over_count here
        
        BaseFolder.manage_afterAdd(self, item, container)
          


registerType(CMFFolder)
